// Generated by CoffeeScript 1.3.3
(function() {
  var Photo, Rembrant, async, baseTemplate, deploy, eco, exec, exif, fs, library, path, rename, spawn, startApp, thumb, _, _ref,
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  fs = require('fs');

  path = require('path');

  _ = require('underscore');

  _ref = require('child_process'), spawn = _ref.spawn, exec = _ref.exec;

  async = require('async');

  eco = require('eco');

  thumb = require('./thumbnail').thumb;

  Photo = require('./photo').Photo;

  exif = require('./exif');

  rename = require('./rename');

  deploy = require('./deploy');

  baseTemplate = fs.readFileSync(__dirname + "/../views/base.html", "utf-8");

  library = {
    source: 'test-photos',
    cache: 'cache',
    lastModified: '',
    awsKey: '',
    awsSecret: '',
    awsBucket: '',
    photos: [],
    albums: [
      {
        id: 1,
        name: 'Unsorted'
      }
    ]
  };

  Rembrant = (function() {

    function Rembrant(filename, cwd) {
      var photo, photos, _i, _len, _ref1;
      this.cwd = cwd;
      if (!fs.existsSync(filename)) {
        console.log("Path doesn't exist, creating...");
        this.createLibrary();
        return;
      }
      this.library = fs.readFileSync(filename, 'utf-8');
      this.library = JSON.parse(this.library);
      photos = [];
      _ref1 = this.library.photos;
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        photo = _ref1[_i];
        photos.push(Photo.fromJSON(photo));
      }
      this.library.photos = photos;
    }

    Rembrant.prototype.addAlbum = function(album) {
      var max;
      max = _.max(this.library.albums, function(item) {
        return item.id;
      });
      this.library.albums.push({
        id: max.id + 1,
        name: album
      });
      return this.save();
    };

    Rembrant.prototype.makeThumbs = function() {
      if (!fs.existsSync(this.library.cache)) {
        fs.mkdirSync(this.library.cache);
      }
      thumb({
        source: this.library.source,
        destination: this.library.cache,
        suffix: '_930',
        concurrency: 4,
        width: 930
      }, function() {
        return console.log('Done processing 800px wide thumbnails');
      });
      return thumb({
        source: this.library.source,
        destination: this.library.cache,
        suffix: '_160',
        concurrency: 4,
        width: 160
      }, function() {
        return console.log('Done processing 160px wide thumbnails');
      });
    };

    Rembrant.prototype.getPhotoById = function(id) {
      var results;
      results = _.filter(this.library.photos, function(photo) {
        return photo.id === id;
      });
      if (results.length === 1) {
        return results[0];
      } else {
        return null;
      }
    };

    Rembrant.prototype.createLibrary = function() {
      var string;
      string = JSON.stringify(library, null, 4);
      return fs.writeFileSync('library.json', string);
    };

    Rembrant.prototype.importPhotos = function() {
      var file, files, libraryFilenames, photos, _i, _len;
      files = fs.readdirSync(this.library.source);
      photos = this.library.photos || [];
      libraryFilenames = _.reject(files, function(file) {
        var e;
        e = path.extname(file);
        return e !== '.jpg';
      });
      for (_i = 0, _len = libraryFilenames.length; _i < _len; _i++) {
        file = libraryFilenames[_i];
        if (__indexOf.call(photos, file) < 0) {
          photos.push(new Photo(file, this.library.source));
        }
      }
      this.library.photos = photos;
      return this.ensureExif();
    };

    Rembrant.prototype.ensureExif = function() {
      var _this = this;
      return async.concatSeries(this.library.photos, exif.readExifImage, function(err, list) {
        console.log(list);
        console.log('done');
        return _this.save();
      });
    };

    Rembrant.prototype.scan = function() {
      var file, files, newFiles, _i, _len;
      files = fs.readdirSync(this.library.source);
      newFiles = _.difference(files, _.pluck(this.library.photos, 'filename'));
      newFiles = _.reject(newFiles, function(file) {
        var e;
        e = path.extname(file);
        return e !== '.jpg';
      });
      for (_i = 0, _len = newFiles.length; _i < _len; _i++) {
        file = newFiles[_i];
        console.log(file);
        this.library.photos.push(new Photo(null, file));
      }
      this.library.photos = _.sortBy(this.library.photos, function(photo) {
        return photo.filename;
      });
      this.save();
      return this.ensureIds();
    };

    Rembrant.prototype.save = function() {
      var string;
      this.library.lastModified = new Date;
      string = JSON.stringify(this.library, null, 4);
      return fs.writeFileSync('library.json', string);
    };

    Rembrant.prototype.imagesByDate = function() {
      return _.sortBy(this.library.photos, function(image) {
        return image.exif['Exif.Photo.DateTimeOriginal'];
      });
    };

    Rembrant.prototype.getSimpleLibrary = function() {
      var photos;
      photos = _.map(this.library.photos, function(i) {
        return {
          filename: i.filename
        };
      });
      return JSON.stringify({
        albums: this.library.albums,
        photos: photos
      });
    };

    Rembrant.prototype.generateIndex = function() {
      var html, template;
      template = fs.readFileSync(__dirname + "/../views/base.html", "utf-8");
      html = eco.render(template, {
        data: this.getSimpleLibrary()
      });
      return fs.writeFileSync(this.cwd + "/build/index.html", html);
    };

    Rembrant.prototype.finalizeFile = function(content, filename) {
      var rendered;
      rendered = eco.render(baseTemplate, {
        content: content
      });
      return fs.writeFileSync(this.cwd + "/build/" + filename, rendered);
    };

    Rembrant.prototype.copyStaticFiles = function() {
      exec("rm -rf " + this.cwd + "/build/static");
      exec("cp -rf " + __dirname + "/../static/js/* " + this.cwd + "/build/.");
      exec("cp -rf " + __dirname + "/../static/css/* " + this.cwd + "/build/.");
      return exec("cp " + this.library.cache + "/*.jpg " + this.cwd + "/build/.");
    };

    Rembrant.prototype.normalize = function() {
      return rename.renameFilesInDirectory(this.library.source, function() {
        return console.log('done');
      });
    };

    Rembrant.prototype.removeExif = function() {
      var lib, p, _i, _len, _ref1;
      lib = [];
      _ref1 = this.library.photos;
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        p = _ref1[_i];
        delete p.exif;
        lib.push(p);
      }
      this.library.photos = lib;
      return this.save();
    };

    Rembrant.prototype.ensureIds = function() {
      var ids, max, p, _i, _j, _len, _len1, _ref1, _ref2;
      _ref1 = this.library.photos;
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        p = _ref1[_i];
        ids = p.id;
      }
      ids = _.compact(ids);
      if (ids.length === 0) {
        max = 1;
      } else {
        max = _.max(ids);
      }
      _ref2 = this.library.photos;
      for (_j = 0, _len1 = _ref2.length; _j < _len1; _j++) {
        p = _ref2[_j];
        p.id = max;
        max++;
      }
      return this.save();
    };

    Rembrant.prototype["export"] = function() {
      if (!fs.existsSync(this.cwd + "/build")) {
        fs.mkdirSync(this.cwd + "/build");
      }
      this.generateIndex();
      return this.copyStaticFiles();
    };

    Rembrant.prototype.deploy = function() {
      return deploy.deploy(this.library, this.cwd);
    };

    return Rembrant;

  })();

  startApp = function(rembrant, importPath) {
    var app, express, importer, routes;
    express = require("express");
    app = express.createServer(express.bodyParser(), express.cookieParser(), express.session({
      secret: "cvfrFRQW352rrvf4132"
    }));
    app.configure(function() {
      this.set('views', __dirname + '/../views');
      this.set('view engine', 'jade');
      app.use(express["static"](__dirname + '/../public'));
      return this.use(this.router);
    });
    routes = {
      index: function(req, res) {
        var html;
        html = fs.readFileSync(__dirname + "/../views/index.eco", "utf-8");
        return res.end(html);
      },
      image: function(req, res) {
        var filename, p;
        filename = req.params.filename;
        p = "" + rembrant.cwd + "/" + rembrant.library.cache + "/" + filename;
        return fs.readFile(p, "binary", function(err, data) {
          return res.end(data, 'binary');
        });
      },
      photos: function(req, res) {
        var photos;
        photos = _.clone(rembrant.library.photos);
        return res.end(JSON.stringify(photos.reverse()));
      },
      updatePhoto: function(req, res) {
        var data, id, photo;
        id = parseInt(req.params.id);
        data = req.body;
        photo = rembrant.getPhotoById(id);
        if (photo) {
          res.send(202);
          photo.albums = data.albums;
          return rembrant.save();
        } else {
          return res.send(404);
        }
      },
      albums: function(req, res) {
        return res.end(JSON.stringify(rembrant.library.albums));
      },
      newAlbum: function(req, res) {
        var name;
        name = req.body.name;
        rembrant.addAlbum(name);
        return res.send(201);
      }
    };
    app.get('/', routes.index);
    app.get('/image/:filename', routes.image);
    app.get('/photos', routes.photos);
    app.put('/photos/:id', routes.updatePhoto);
    app.get('/albums', routes.albums);
    app.post('/albums', routes.newAlbum);
    if (importPath) {
      importer = require('./importer').main(app, importPath);
    }
    console.log('Serving http://localhost:8888');
    return app.listen(8888);
  };

  exports.runFromCli = function() {
    var imagePath, program, rembrant;
    program = require('commander');
    program.version('0.0.1');
    program.option('-i, --import [path]');
    program.option('-s, --scan', 'Check if new images were added');
    program.option('-e, --export', 'Generate deployable html structure');
    program.option('-d, --deploy', 'Deploy files to CDN');
    program.option('-t, --thumbs', 'Create thumbnails for all files');
    program.option('-n, --rename', 'Rename files in your library');
    program.option('-r, --serve', 'Run the rembrant server');
    program.option('-a, --add-album <name>', 'Add an album to library');
    program.option('-m, --exif', 'Kill exif');
    program.option('-x, --ids', 'Ensure IDs');
    program.parse(process.argv);
    rembrant = new Rembrant('library.json', process.cwd());
    if (program.thumbs) {
      rembrant.makeThumbs();
    }
    if (program.scan) {
      rembrant.scan();
      rembrant.makeThumbs();
    }
    if (program["export"]) {
      rembrant["export"]();
    }
    if (program.deploy) {
      rembrant.deploy();
    }
    if (program["import"] && program["import"] === true) {
      rembrant.importPhotos();
    }
    if (program["import"] && program["import"] !== true) {
      console.log('importing');
      imagePath = program["import"];
      startApp(rembrant, imagePath);
    }
    if (program.rename) {
      rembrant.normalize();
    }
    if (program.serve) {
      startApp(rembrant);
    }
    if (program.addAlbum) {
      rembrant.addAlbum(program.addAlbum);
    }
    if (program.exif) {
      rembrant.removeExif();
    }
    if (program.ids) {
      return rembrant.ensureIds();
    }
  };

}).call(this);
